<?php

/**
 * Cryptor class
 *
 * @author Marshall
 */
class cryptor {

    protected static $addon = 'cryptor';
    protected static $plugin = null;
    protected static $cipher = null;
    protected static $key = null;

    /**
     * Initialize the class
     */
    public static function init() {
        static::$cipher = rex_addon::get('cryptor')->getConfig('cipher');
        static::$key = rex_addon::get('cryptor')->getConfig('key');
    }

    /**
     * Encrypt given data
     * @param <mixed> $data
     * @return <mixed> $encryptredData
     */
    public static function encrypt($data = '', $key = null) {
        if (is_array($data)) {
            $array = [];
            foreach ($data as $key => $value) {
                $array[$key] = self::encrypt($value, $key);
            }
            return $array;
        } else if (is_object($data)) {
            $object = new $data;
            foreach ($data as $key => $value) {
                $object->$key = self::encrypt($value, $key);
            }
            return $object;
        }

        return self::_encrypt($data, $key);
    }

    /**
     * Decrypt given data
     * @param <mixed> $data
     * @return <mixed> $decryptredData
     */
    public static function decrypt($data = '', $key = null) {
        if (is_array($data)) {
            $array = [];
            foreach ($data as $key => $value) {
                $array[$key] = self::decrypt($value, $key);
            }
            return $array;
        } else if (is_object($data)) {
            $object = new $data;
            foreach ($data as $key => $value) {
                $object->$key = self::decrypt($value, $key);
            }
            return $object;
        }

        return self::_decrypt($data, $key);
    }

    /**
     * Returns true if cipher exists
     * @param <string> $cipher
     * @return <boolean>
     */
    public static function hasCipher($cipher) {
        return in_array($cipher, openssl_get_cipher_methods());
    }

    /**
     * Private encryptor
     * @param <string> $string
     * @return <string>
     */
    private static function _encrypt($string, $key) {
        if (empty($key)) {
            $key = static::$key;
        }

        // For php version lower 7.1
        if (version_compare(phpversion(), '7.1', '<')) {
            $iv = self::iv();
            $raw = openssl_encrypt($string, static::$cipher, $key, OPENSSL_RAW_DATA, $iv);
            $hmac = hash_hmac('sha256', $raw, $key, true);
            return base64_encode($iv . $hmac . $raw);
        }

        return openssl_encrypt($string, static::$cipher, $key, 0, self::iv());
    }

    /**
     * Private decryptor
     * @param <string> $string
     * @return <string>
     */
    private static function _decrypt($string, $key) {
        if (empty($key)) {
            $key = static::$key;
        }

        // For php version lower 7.1
        if (version_compare(phpversion(), '7.1', '<')) {

            $c = base64_decode($string);
            if (empty($c)) {
                return $string;
            }
            $ivlen = openssl_cipher_iv_length(static::$cipher);
            $iv = substr($c, 0, $ivlen);
            $sha2len = 32;
            $hmac = substr($c, $ivlen, $sha2len);
            $raw = substr($c, $ivlen + $sha2len);

            // PHP 5.6+ timing attack safe comparison
            $calcmac = hash_hmac('sha256', $raw, $key, true);
            if (hash_equals($hmac, $calcmac)) {
                return (string) openssl_decrypt($raw, static::$cipher, $key, OPENSSL_RAW_DATA, $iv);
            }
            return $string;
        }

        return openssl_decrypt($string, static::$cipher, $key, 0, self::iv());
    }

    /**
     * Private: returns an initialization vector
     * @return <string>
     */
    private static function iv() {
        $ivlen = openssl_cipher_iv_length(static::$cipher);
        return openssl_random_pseudo_bytes($ivlen);
    }

}
